/* All Rights Reserved, Copyright (c) FUJITSU LIMITED 1998 */

#include <iostream>
#include <stdlib.h>
#include <string.h>
#ifdef OM_PC
#include <windows.h>
#endif /* OM_PC */


using std::cout;
using std::endl;

#include "orb_cplus.h"

#include "simple.H"

/* -------------------------------------------------------------------- */

static	CORBA::ORB_ptr					orb;
static	CORBA::BOA_ptr					boa;
static	FJ::ImplementationRep_ptr		impl_rep;
static	CORBA::ImplementationDef_ptr	impl;

/* -------------------------------------------------------------------- */

void
print_any( const CORBA::Any &any )
{
	CORBA::TypeCode_ptr	_tc;
	CORBA::Environment	env;

	_tc = any.type();
	if( _tc->equal( _tc_ODsample_sample1, env ) ) {
		ODsample::sample1	*smp1;
		smp1 = (ODsample::sample1 *)any.value();
		cout << "smp1->para1 = " << smp1->para1;
		cout << "smp1->para2 = " 
			<< (const CORBA::Char *)smp1->para2 << endl;
	}
	if( _tc->equal( _tc_ODsample_sample2, env ) ) {
		ODsample::sample2	*smp2;
		smp2 = (ODsample::sample2 *)any.value();
		cout << "smp2->para1 = " << smp2->para1 << endl;
		cout << "smp2->para2 = " << smp2->para2 << endl;
	}
	if( _tc->equal( _tc_ODsample_sample3, env ) ) {
		ODsample::sample3	*smp3;
		smp3 = (ODsample::sample3 *)any.value();
		cout << "smp3->para1 = " << smp3->para1 << endl;
		cout << "smp3->para2 = " << smp3->para2 << endl;
	}
}

CORBA::Any*
ODsample_anytest_impl::op1( 
	const	CORBA::Any	&any1,
	CORBA::Any*&	any2,
	CORBA::Any&	any3,
	CORBA::Environment & )
	throw( CORBA::Exception )
{
	print_any( any1 );
	print_any( any3 );

	/* out parameter set */
	ODsample::sample1 *smp1 = new ODsample::sample1;
	smp1->para1 = 100;
	CORBA::Char *str = CORBA::string_alloc(4);
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	strcpy_s( str, 4, "OUT" );
#else
	strcpy( str, "OUT" );
#endif
	smp1->para2 = str;
	any2 = new CORBA::Any( _tc_ODsample_sample1, smp1, CORBA_TRUE );

	/* inout parameter set */
	ODsample::sample2 *smp2 = new ODsample::sample2;
	smp2->para1 = 'x';
	smp2->para2 = (float)0.01;
	any3.replace( _tc_ODsample_sample2, smp2, CORBA_TRUE );

	/* return set */
	ODsample::sample3 *smp3 = new ODsample::sample3;
	smp3->para1 = 'z';
	smp3->para2 = 0.001;
	CORBA::Any *any = 
		new CORBA::Any( _tc_ODsample_sample3, smp3, CORBA_TRUE );

	return any;
}
	
/* -------------------------------------------------------------------------- */

int
main( int argc, char *argv[] )
{
	int	current_argc = argc;

	CORBA::Object_ptr	o;
	CORBA::Environment_ptr	env;

	env = new CORBA::Environment;

	try {
		orb = CORBA::ORB_init( current_argc, argv, FJ_OM_ORBid, *env );

		boa = orb->BOA_init( current_argc, argv, CORBA_BOA_OAid, *env );

		o = orb->resolve_initial_references(
				CORBA_ORB_ObjectId_ImplementationRepository, 
				*env );

		if ( o == NULL ){
			fprintf( stderr, "Not Found ImplementationRepository\n" );
			return 1;
		}

		impl_rep = FJ::ImplementationRep::_narrow( o );
		CORBA::release( o );

		o = impl_rep->lookup_id( _IMPL_ODsample_anytest, *env );
		impl = CORBA::ImplementationDef::_narrow( o );
		CORBA::release( o );

		boa->impl_is_ready( impl, *env );
	}
	catch ( CORBA::SystemException &se ) {
		printf("SystemException raised!\n");
		printf("exception-id = %s\n", se.id());
		printf("minor = 0x%x\n", se.minor());
		return 1;
	}
	catch ( CORBA::Exception &e ) {
		printf("Exception raised!\n");
		printf("exception-id = %s\n", e.id());
		return 1;
	}

	delete env;

	return 0;
}

